#------------------------------------------------------------------------------------------------
# Permission to use, copy, modify, and distribute this software for 
# any purpose and without fee is hereby granted.
#------------------------------------------------------------------------------------------------
# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
# WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#------------------------------------------------------------------------------------------------
# THIS PLUGIN IS ANOTHER FIGMENT OF MY IMAGENATION AND IS NOT BASED ON OR COMPLY WITH ANYTHING! 
#------------------------------------------------------------------------------------------------
# IT WAS CREATED AND ONLY TESTED IN A WINDOWS ENVIRONMENT AND MAY NOT FUNCTION ON A MAC.
#------------------------------------------------------------------------------------------------
#    Name:	Quad Chamfer
#      By:	sdmitch
#   Usage:	Plugins>SDM Tools>Misc Tool>Quad Chamfer
#		 Note:	
#			
#    Date:	
#------------------------------------------------------------------------------------------------
require 'Sketchup'
# ------------------  MENU SETUP ---------------------- #
unless $sdm_tools_menu
	$sdm_tools_menu = UI.menu("Plugins").add_submenu("SDM Tools")
	$sdm_Edge_tools = $sdm_tools_menu.add_submenu("Edge Tool")
	$sdm_Face_tools = $sdm_tools_menu.add_submenu("Face Tool")
	$sdm_CorG_tools = $sdm_tools_menu.add_submenu("CorG Tool")
	$sdm_Misc_tools = $sdm_tools_menu.add_submenu("Misc Tool")
end
unless file_loaded?(__FILE__)
	$sdm_Misc_tools.add_item("Quad Chamfer") { Sketchup.active_model.select_tool SDM::Quad_Chamfer.new }
	file_loaded(__FILE__)
end
# ------------------------------------------------------ #
module SDM

	class Quad_Chamfer

		def initialize
			@mod=Sketchup.active_model
			@ent=@mod.active_entities
			@sel=@mod.selection
			@vue=@mod.active_view
			@ip=Sketchup::InputPoint.new
			self.reset
		end
		
		def reset
			@sel.clear
			@edges = []
			@status = 0
			@status_text = "Pick edge #{@status+1} of six edges that define the three planes"
		end
		
		def onMouseMove(flags, x, y, view)
			@ip.pick view,x,y; view.tooltip = @ip.tooltip; view.refresh
			Sketchup::set_status_text @status_text,SB_PROMPT
			Sketchup::set_status_text "", SB_VCB_LABEL
			Sketchup::set_status_text "", SB_VCB_VALUE
		end
		
		def onLButtonDown(flags, x, y, view)
			ph = view.pick_helper; ph.do_pick x,y; best=ph.best_picked
			if best.is_a?(Sketchup::Edge)
				@edges << best; @status += 1
				@status_text = "Pick edge #{@status+1} of six edges that define the three planes"
				@status_text = "Left click to proceed, Right to start over" if @status==6
			elsif @status==6
				@mod.start_operation "Quad Chamfer"
				pt,p0,p1,p2 = self.quad_chamfer
				edg=@ent.add_line(pt,p0)
				edg=@ent.add_line(pt,p1);edg.find_faces
				edg=@ent.add_line(pt,p2);edg.find_faces
				@mod.commit_operation
				self.reset
			else
				UI.beep
			end
		end
		
		def onRButtonDown(flags, x, y, view)
			@status==0 ? onCancel(flags,view) : self.reset
		end
		
		def onCancel(flags,view)
			Sketchup.send_action "selectSelectionTool:"
		end
		 
		def draw(view)
			if( @ip.valid? && @ip.display? )
				@ip.draw(view)
			end
			if @ip.edge
				case @status
					when 0,1 then view.drawing_color='red'
					when 2,3 then view.drawing_color='blue'
					when 4,5 then view.drawing_color='green'
				end
				view.line_width=5
				view.draw_line @ip.edge.start.position,@ip.edge.end.position
			end
			view.line_width=5
			view.drawing_color='red'
			view.draw_line @edges[0].vertices.map{|v|v.position} if @status>0
			view.draw_line @edges[1].vertices.map{|v|v.position} if @status>1
			view.drawing_color='blue'
			view.draw_line @edges[2].vertices.map{|v|v.position} if @status>2
			view.draw_line @edges[3].vertices.map{|v|v.position} if @status>3
			view.drawing_color='green'
			view.draw_line @edges[4].vertices.map{|v|v.position} if @status>4
			view.draw_line @edges[5].vertices.map{|v|v.position} if @status>5
			if @status==6
				pt,p0,p1,p2 = self.quad_chamfer
				view.drawing_color='yellow'
				view.draw_line pt,p0
				view.draw_line pt,p1
				view.draw_line pt,p2
			end
		end
		
		def quad_chamfer
				pv0 = @edges[0].vertices+@edges[1].vertices;pv0.flatten!;pv0.uniq!; 
				cp0 = (@edges[0].vertices&@edges[1].vertices)[0].position
				nrm = @edges[0].line[1]*@edges[1].line[1]; pl0 = [cp0,nrm];

				pv1 = @edges[2].vertices+@edges[3].vertices;pv1.flatten!;pv1.uniq!;
				cp1 = (@edges[2].vertices&@edges[3].vertices)[0].position
				nrm = @edges[2].line[1]*@edges[3].line[1]; pl1 = [cp1,nrm];

				pv2 = @edges[4].vertices+@edges[5].vertices;pv2.flatten!;pv2.uniq!;
				cp2 = (@edges[4].vertices&@edges[5].vertices)[0].position
				nrm = @edges[4].line[1]*@edges[5].line[1]; pl2 = [cp2,nrm];

				line = Geom::intersect_plane_plane(pl0, pl1)
				point = Geom::intersect_line_plane(line, pl2)
				cv0 = (pv0 & pv1)[0].position; 
				cv1 = (pv1 & pv2)[0].position; 
				cv2 = (pv2 & pv0)[0].position; 
				return point,cv0,cv1,cv2
		end
		
	end
	
end
